import numpy as np
import laspy
import utm # src: https://github.com/Turbo87/utm
from itertools import groupby
from server.data_processing.area_processing import calculate_corners, define_gridareas


# hard coded files for test data
lazData_path = ["server/example_lidar_data/ot_N_000005_1.laz", "server/example_lidar_data/ot_N_000033_1.laz"]

# Info about data
with laspy.open(lazData_path[0]) as fh:
    # Print metadata properties
    print("File Version:", fh.header.version)
    print("Point Count:", fh.header.point_count)
    print("Scale Factors:", fh.header.scale)
    print("Offset:", fh.header.offset)

    print('Points from Header:', fh.header.point_count)
    las = fh.read()
    print(las)
    print('Points from data:', len(las.points))
    ground_pts = las.classification == 2
    bins, counts = np.unique(las.return_number[ground_pts], return_counts=True)
    print('Ground Point Return Number distribution:')
    for r, c in zip(bins, counts):
        print('    {}:{}'.format(r, c))

# check if lidar points is within range of the area selected
def inArea(position, areaRange):
    x, y, _ = position
    print(areaRange[0][0], " > ", x, " > ", areaRange[1][0], ") and (", areaRange[0][1], " < ", y, " < ", areaRange[1][1])
    if (areaRange[0][0] > x > areaRange[1][0]) and (areaRange[0][1] < y < areaRange[1][1]):
        print("inside area")
        return True
    else:
        return False

# Calculation of height in an area from las files
def find_height(points):
    height_differences = [] # final container for height data

    # sort the points
    sorted_coords = sorted(points, key=lambda coord: (coord[0],coord[1]))

    # group the sorted points that has the same xy- coordinates together
    groupCoords = [list(group) for key, group in groupby(sorted_coords, key=lambda coord: (coord[0], coord[1]))]

    # loop through the groups to find the difference in height
    for group in groupCoords:
        if len(group) < 2 or group[0] == group[1]:
            continue # jump over iteration if there is only one coordinate or lidar registered the same point twice

        # find max and min height
        min_height = min(coords[2] for coords in group)
        max_height = max(coords[2] for coords in group)

        # difference between them
        difference = max_height - min_height
        height_differences.append(difference)

    # list of thickness in a area
    return height_differences

# find the height of an area based on the coordinates of it's center
# and it's affiliations (subId and groupId) (soon to be implemented
def calculate_area_data(center):
    # container for all the heights in area
    area_heights = []

    # Limit the data to specific areas

    # this is only temporary data for coordinates in arctic
    # areas = [
    #     [(-3671212, 7898422), (-3200175, 4699978)],
    #     [(60.815356, 10.672022), (60.774878, 10.768867)],
    # ]

    # specified the area and it's range
    #area = (area[0], area[2])
    #print("area size")
    # NB: is only for the test data (soon to be removed and switched with center)
    #areazone = ((((7898422) - (4699978)))/2.0 + 4699978), (((-3671212) - (-3200175)))/2.0 - 3200175
    #areazone = ((-3671212 - (-3200175))/center[0] - 3200175, (7898422 - 4699978)/center[1] + 4699978)

    #print(areazone)


    # max and min initial (-9.790000, -615.719971) - (696.619995, 96.660004)
    # max and min g(idk)  (-36709.790000, 78284.280029) - (-36003.380005,78996.660004)
    # should be within (-3200175, 4699978) - (-3671212, 7898422)
    #print(utm.to_latlon(4699978, -3200175,18,'S'))


    #areazone = ((((-3200175 + 3671212) / ( area[0][0] - area[2][0]))),
    #                ((7898422 - 4699978) / (area[0][1] - area[2][1])))
    #areazone = ((((-3200175 / area[0][0]) - ( - 3671212 / area[2][0]))),
    #                ((7898422 / area[0][1]) - (area[2][1] / 4699978)))

    #area = calculate_corners(center[0],center[1], 1500)

    areazone = calculate_corners(3435693.5,7550000.0, 20000000000)
    #print(area[0]," area ", area[2])

    #start_point = areazone[0]
    #areazone = (areazone[2][0] - areazone[0][0], areazone[2][1] - areazone[0][1])
    #print("az",areazone)

    # areazone = [(((center[0]) * ((-3671212 / 60.815356) - (-3200175 / 60.774878))) - 3200175,
    #              (center[1] * ((7898422 / 10.768867) - (4699978 / 10.672022))) + 4699978),
    #             (((center[0] - 100) * ((-3671212 / 60.815356) - (-3200175 / 60.774878))) - 3200175,
    #              ((center[1] - 1) * ((7898422 / 10.768867) - (4699978 / 10.672022))) + 4699978)]

    # Refactor lidar data to a readable format
    iceOver = laspy.read(lazData_path[0])
    iceUnder = laspy.read(lazData_path[1])

    # add two files together temporary test data(soon to be removed)
    ice_points = list(zip(iceOver.X,iceOver.Y,iceOver.Z)) + list(zip(iceUnder.X,iceUnder.Y,iceUnder.Z))
    # area height in all the subsections
    #grid_area_heights = define_gridareas(-3435693.5,6299200.0, 20000000000,4)
    grid_area_heights = define_gridareas(3435693.5,7550000.0, 20000000000,4)

    print("",grid_area_heights)

    # find the heights of each sub-area => area-heights
    for sub_area in grid_area_heights:

        #main_vector = tuple(x - y for x, y in zip(area[2], area[0]))
        #sub_vector = tuple(x - y for x, y in zip(sub_area[2][2], sub_area[2][0]))
        #relative_to_laz = (tuple(x - y for x, y in zip(sub_area[2][0], area[0])),
        #                   tuple(x + y for x, y in zip(sub_vector, sub_area[2][0])))
        #relative_to_laz = (tuple(x / y for x, y in zip(relative_to_laz[0], main_vector)),
        #                   tuple(x / y for x, y in zip(relative_to_laz[1], sub_area[2][0])))
        #relative_to_laz = (tuple(x * y for x, y in zip(relative_to_laz[0], areazone)),
        #                   tuple(x * y for x, y in zip(relative_to_laz[1], areazone)))
        #relative_to_laz = (tuple(x + y for x, y in zip(relative_to_laz[0], start_point)),
        #                   tuple(x + y for x, y in zip(relative_to_laz[1], start_point)))

        #print("areaZone",areazone)
        #print("sub area",sub_area)
        #print("relativer",relative_to_laz)

        #zone_limit = (tuple(x * y for x, y in zip(relative_to_laz, areazone)),
        #            tuple(x * y for x, y in zip(sub_area[2][2], areazone)))
        #print("zone_limit",zone_limit)
        #zone_limit = (tuple(x + y for x, y in zip(zone_limit[0], start_point)),
        #            tuple(x + y for x, y in zip(zone_limit[1], start_point)))

        #print("zone_limit",zone_limit)
        print(type(ice_points[0]))
        ice_points = list(filter(lambda point_position: inArea(point_position, ((int(-sub_area[2][0][0]),int(sub_area[2][0][1])),(int(-sub_area[2][2][0]),int(sub_area[2][2][1])))), ice_points))
        #ice_points = list(filter(lambda point_position: inArea(point_position, (((-areazone[0][0]),areazone[0][1]),(-areazone[2][0],areazone[2][1]))), ice_points))
        #ice_points = list(filter(lambda point_position: inArea(point_position, (((-3200000,7400000),(-3671212,7898422)))), ice_points))
        print("ice",ice_points)
        area_heights.append((sub_area[0],sub_area[1],find_height(ice_points)))

    return area_heights

print(-3>-4)
print(calculate_area_data((60.0,10.0)))



